<?php
/* WORKFLOWS MANAGER PHP CLASS */
// +----------------------------------------------------------------------+
// | E-XOOPPORT V 2.0                                                     |
// +----------------------------------------------------------------------+
// | Copyright (c) 2004 The E-Xoopport Project                            |
// +----------------------------------------------------------------------+
// | Authors: Redpitt <redpitt@e-xoopport.it>                             |
// +----------------------------------------------------------------------+
//

if (!defined("XOOPS_WFLOWS_INCLUDED")) {
	define("XOOPS_WFLOWS_INCLUDED", 1);
	
	class WorkflowsManager {
	
		// File handlers for workflows and operations log files.
		var $wf_filehandler;
		var $wf_log_filehandler;
		
		// Files status... initially are both closed.
		var $wf_filestatus = 'closed';
		var $wf_logfilestatus = 'closed';
		
		// These 2 variables will store the path of the files.
		var $wf_file;
		var $wf_log_file;
		
		// These arrays will store all workflows and operation log files lines.
		var $wf_array;
		var $wf_log_array;
	
		// Constructor... store files path into variables...
		function WorkflowsManager() {
			$this->wf_file = XOOPS_ROOT_PATH.'/cache/workflows/workflows.php';
			$this->wf_log_file = XOOPS_ROOT_PATH.'/cache/workflows/workflows.log.php';
		}
		
		// This functions opens the workflows file...
		// The argument 'mode' indicates in which mode the
		// file should be opened. 'r' for reading, 'w' for writing
		// 'a' for appending. For further detail on the mode a file can
		// be opened please refer to PHP documentation.
		function OpenWorkflowsFile($mode='r') {
			if ( $this->wf_filestatus == 'opened' ) {
				$this->CloseWorkflowsFile();
			}
			
			if ( !$this->wf_filehandler = @fopen($this->wf_file, $mode) ) {
				return false;
			} else {
				if ( $mode == 'w' || $mode == 'w+' ) {
					if ( @fwrite($this->wf_filehandler, "<?php die(); ?>\n") == -1 ) {
						$this->CloseWorkflowsFile();
						return false;
					}
				}
				
				$this->wf_filestatus = 'opened';
				return true;
			}
		}

		// This function adds a new workflow entry in the file...
		// The argument 'timestamp' is the UNIX timestamp that
		// indicates date and hour of the workflow.
		// 'query' is the SQL instruction to be executed and
		// 'module' is the module id of the module in which the workflow
		// should be executed... if 0, workflow will be executed anywhere.
		function AddLineToWorkflowsFile($timestamp, $query, $module) {
			$query = eregi_replace("/(\015)|(\012)|(\011)/m", "{linebreak}", $query);
			$line = "";
			$line = "".intval($timestamp)."|".$query."|".intval($module)."\n";
			if ( @fwrite($this->wf_filehandler, $line) != -1 ) {
				return true;
			} else {
				return false;
			}
		}
		
		// This function closes the workflows file
		function CloseWorkflowsFile() {
			@fclose($this->wf_filehandler);
			$this->wf_filestatus = 'closed';
		}
		
		// This functions opens the workflows log file...
		// The argument 'mode' indicates in which mode the
		// file should be opened. 'r' for reading, 'w' for writing
		// 'a' for appending. For further detail on the mode a file can
		// be opened please refer to PHP documentation.
		function OpenWorkflowsLogFile($mode='r') {
			if ( $this->wf_logfilestatus == 'opened' ) {
				$this->CloseWorkflowsLogFile();
			}
		
			if ( !$this->wf_log_filehandler = @fopen($this->wf_log_file, $mode) ) {
				return false;
			} else {
				if ( $mode == 'w' || $mode == 'w+' ) {
					if ( @fwrite($this->wf_log_filehandler, "<?php die(); ?>\n") == -1 ) {
						$this->CloseWorkflowsLogFile();
						return false;				
					}
				}
				
				$this->wf_logfilestatus = 'opened';
				return true;
			}
		}

		// This function adds a new workflow log entry in the file...
		// The argument 'timestamp' is the UNIX timestamp that
		// indicates date and hour when the workflow was executed.
		// 'query' is the SQL instruction executed and
		// 'module' is the module id of the module in which the workflow
		// was launched... status indicates if there were errors or not
		// during its execution.	
		function AddLineToLogFile($timestamp, $query, $module, $status) {			
			$query = eregi_replace("/(\015)|(\012)|(\011)/m", "{linebreak}", $query);
			$line2 = "";
			$line2 = "".intval($timestamp)."|".$query."|".intval($module)."|".intval($status)."\n";
			if ( @fwrite($this->wf_log_filehandler, $line2) != -1 ) {
				return true;
			} else {
				return false;
			}
		}
		
		// Closes the workflows log file.
		function CloseWorkflowsLogFile() {
			@fclose($this->wf_log_filehandler);
			$this->wf_logfilestatus = 'closed';
		}

		// Fill in an array which contains all the lines
		// in the workflows file.
		function getWorkflowsArray() {
			$this->wf_array = array();
			
			if ( $retarr = @file($this->wf_file) ) {
				// This process removes the first line of the workflows file...
				$counter = 0;
				foreach ( $retarr as $row ) {
					if ( $counter > 0 ) {
						$this->wf_array[] = $row;
					}
					$counter++;
				}
												
				return $this->wf_array;
			} else {
				return false;
			}
		}
		
		// Fill in an array which contains all the lines
		// in the workflows log file.
		function getWorkflowsLogArray() {
			$this->wf_log_array = array();
			
			if ( $retarr = @file($this->wf_log_file) ) {
				// This process removes the first line of the workflows file...
				$counter = 0;
				foreach ( $retarr as $row ) {
					if ( $counter > 0 ) {
						$this->wf_log_array[] = $row;
					}
					$counter++;
				}
												
				return $this->wf_log_array;
			} else {
				return false;
			}
		}
		
		// Simply call this function to execute all workflows!
		// This function will determine which workflows execute.
		function ExecAllWorkflows() {
			global $xoopsModule, $db;
			
			// Get all workflows to execute...
			$allwflows = $this->getWorkflowsArray();
			// Check if there are workflows to execute.
			if ( !@is_array($allwflows) || count($allwflows) == 0 ) {
				return;
			}
			
			// Stores not executed workflows to rewrite the file later.
			$notmade = array();
			// The actual UNIX Timestamp
			$actual_time = time();
			
			// Opening the workflows log file to write operations.
			$this->OpenWorkflowsLogFile('a');
			
			foreach ( $allwflows as $wf ) {
				// Process each workflow, only if it is the time and in the appropriate module.
				$wf_arr = explode('|', $wf);
				if ( ( intval($wf_arr[0]) <= $actual_time ) && ( ( $xoopsModule && intval($wf_arr[2]) == $xoopsModule->mid() ) || ( intval($wf_arr[2]) == 0 ) ) && count($wf_arr) == 3 ) {
					$myquery = eregi_replace("{linebreak}", "\n", $wf_arr[1]);
					if ( $result = @$db->query($myquery) ) {
						$this->AddLineToLogFile($actual_time, $wf_arr[1], $wf_arr[2], 0);
					} else {
						$this->AddLineToLogFile($actual_time, $wf_arr[1], $wf_arr[2], 1);					
					}
				} else {
					$notmade[] = $wf;
				}
			}
			
			// Rewrite the workflows file and close the log file.
			$this->CleanWorkflowsFile($notmade);		
			$this->CloseWorkflowsLogFile();
		}
		
		// This function deletes old (already-executed) workflows from the file.
		function CleanWorkflowsFile($notmade_lines) {			
			$this->OpenWorkflowsFile('w');
			foreach ( $notmade_lines as $ln ) {
				@fwrite($this->wf_filehandler, $ln);
			}
			$this->CloseWorkflowsFile();
		}
		
		// This function deletes all log entries from the log file.
		function ClearWorkflowsLogFile() {
			$this->OpenWorkflowsLogFile('w');
			$this->CloseWorkflowsLogFile();
		}
		
		// This function counts all workflows...
		function countWorkflows() {
			$wfs = $this->getWorkflowsArray();
			if ( !@is_array($wfs) ) {
				return 0;
			} else {
				return count($wfs);
			}
		}
		
		// This function prints out on-screen workflow planning form
		function renderWorkflowForm($mixedc="") {
			if ( !empty($mixedc) && is_array($mixedc) && count($mixedc) == 6 ) {
				$formtime = mktime($mixedc[0], $mixedc[1], $mixedc[2], $mixedc[3], $mixedc[4], $mixedc[5]);
				$wfstatus = "checked";
			} else {
				$formtime = time();
				$wfstatus = "unchecked";
			}
						
			$monthsarray = array(_JANUARY, _FEBRUARY, _MARCH, _APRIL, _MAY, _JUNE, _JULY, _AUGUST, _SEPTEMBER, _OCTOBER, _NOVEMBER, _DECEMBER);
		
			$ret = '<input type="checkbox" class="checkbox" name="wf_enabled" id="wf_enabled" '.$wfstatus.'> '._WF_ENABLE.'</input><br /><br />
			'._MONTH.': <select class="select" name="wf_month" id="wf_month">';
			
			for ($mnt=1; $mnt<=12; $mnt++) {
				$mnt < 10 ? $mnt = "0".$mnt."" : $mnt = "".$mnt."";
				
				$ret .= '<option value="'.$mnt.'"';
				if ( intval($mnt) == intval(date("m", $formtime)) ) {
					$ret .= ' selected';
				}
				$ret .= '>'.$monthsarray[intval($mnt) - 1].'</option>';
			}
			
			$ret .= '</select> '._DAY.': <select class="select" name="wf_day" id="wf_day">';
			
			for ($day=1; $day <= 31; $day++) {
				$day < 10 ? $day = "0".$day."" : $day = "".$day."";
				
				$ret .= '<option value="'.$day.'"';
				if ( intval($day) == intval(date("d", $formtime)) ) {
					$ret .= ' selected';
				}
				$ret .= '>'.$day.'</option>';
			}
			
			$ret .= '</select> '._YEAR.': <select class="select" name="wf_year" id="wf_year">';
			
			$currentyear = intval(date("Y", time()));
			$selyear = intval(date("Y", $formtime));
			
			for ($yr=$currentyear; $yr<=$currentyear+4; $yr++) {
				$yr = "".$yr."";
				$ret .= '<option value="'.$yr.'"';
				if ( intval($yr) == $selyear ) {
					$ret .= ' selected';
				}
				
				$ret .= '>'.$yr.'</option>';
			}
			
			$ret .= '</select> '._HOURS.': <select class="select" name="wf_hours" id="wf_hours">';
			
			for ($hr=1; $hr <= 24; $hr++) {
				$hr < 10 ? $hr = "0".$hr."" : $hr = "".$hr."";
				$ret .= '<option value="'.$hr.'"';
				
				if ( intval($hr) == intval(date("H", $formtime)) ) {
					$ret .= ' selected';
				}
				
				$ret .= '>'.$hr.'</option>';
			}
			
			$ret .= '</select> '._MINUTES.': <select class="select" id="wf_minutes" name="wf_minutes">';
			
			for ($mins=0; $mins <= 59; $mins++) {
				$mins < 10 ? $mins = "0".$mins."" : $mins = "".$mins."";
				$ret .= '<option value="'.$mins.'"';
				
				if ( intval($mins) == intval(date("i", $formtime)) ) {
					$ret .= ' selected';
				}
				
				$ret .= '>'.$mins.'</option>';
			}
			
			$ret .= '</select> '._SECONDS.': <select class="select" id="wf_seconds" name="wf_seconds">';
			
			for ($secs=0; $secs <= 59; $secs++) {
				$secs < 10 ? $secs = "0".$secs."" : $secs = "".$secs."";
				$ret .= '<option value="'.$secs.'"';
				
				if ( intval($secs) == intval(date("s", $formtime)) ) {
					$ret .= ' selected';
				}
				
				$ret .= '>'.$secs.'</option>';
			}
			
			$ret .= '</select>';
			
			return $ret;
		}
		
		function getTimestampFromForm() {
			$d = $_REQUEST['wf_day'];
			$m = $_REQUEST['wf_month'];
			$y = $_REQUEST['wf_year'];
			
			$h = $_REQUEST['wf_hours'];
			$mn = $_REQUEST['wf_minutes'];
			$s = $_REQUEST['wf_seconds'];

			if ( $_REQUEST['wf_enabled'] == 'on' ) {
				return @mktime($h, $mn, $s, $m, $d, $y);
			} else {
				return time();
			}
		}

	}
}
?>