<?php
/* XML/XSLT TRANSFORMATION PHP CLASS */
// +----------------------------------------------------------------------+
// | E-XOOPPORT RC 1.0                                                    |
// +----------------------------------------------------------------------+
// | Copyright (c) 2004 The E-Xoopport Project                            |
// +----------------------------------------------------------------------+
// | Authors: Redpitt <redpitt@e-xoopport.it>                             |
// +----------------------------------------------------------------------+
// | IMPORTANT NOTICES: this class works correctly only if the PHP XSLT   |
// | library is installed! This class is here to give functionality to    |
// | some modules that can export database content in HTML or for modules |
// | that have the need to build xml files... This class offers basic     |
// | functionalities only! PHP 4 support for XML/XSLT is too limited!     |
// +----------------------------------------------------------------------+
// | For more information about this class and for documentation, please  |
// | see the "manual" folder...                                           |
// +----------------------------------------------------------------------+
//

class XoopsXML {

	// Predefined variables
	var $xml_elements = array();
	var $xml_attributes = array();
	var $xml_buffer = "";
	var $root_element = "";
	var $xml_template = "";
	var $xml_encoding = "ISO-8859-1";
		
	// Constructor... if "$template" is specified, a link to an xsl stylesheet will be
	// included in the result file.
	function XoopsXML($template="", $encoding="") {
		if ( !empty($template) ) {
			$this->xml_template = $template;
		}
		
		if ( !empty($encoding) ) {
			$this->xml_encoding = $encoding;
		}
	}
	
	// Adds the main xml tree element... it could not be removed!
	// Usage: $xmlclass->addElement("MyRootElement");
	function addRootElement($elementName) {
		$this->root_element = $elementName;
	}
		
	// Adds an attribute to the selected element...
	// Usage: $xmlclass->addAttribute("ElementName", "MyAttribute", "Value of MyAttribute");
	function addAttribute($elementName, $attributeName, $attributeValue) {
		$this->xml_attributes[$elementName][$attributeName] = $attributeValue;
	}
	
	// Removes an attribute from the specified element...
	// Usage: $xmlclass->removeAttribute("ElementName", "MyAttribute");
	function removeAttribute($elementName, $attributeName) {
		unset($this->xml_attributes[$elementName][$attributeName]);
	}
	
	// Adds an xml element subordinated to the root element...
	// ...or to another one.
	// Usage: $xmlclass->addElement("RelativeNode", "ElementName", "ElementValue");
	function addElement($relativeAtNode="", $subEleName, $subEleVal="") {
		$this->xml_elements[$subEleName] = array($subEleName, $subEleVal, $relativeAtNode);
	}
	
	// Removes an element from the array.
	// Usage: $xmlclass->removeElement("ElementName");
	function removeElement($subEleName) {
		unset($this->xml_elements[$subEleName]);
	}
	
	// Returns true if a node has to be written after another one... false if not. For internal use!
	function isSubordinate($node) {
		if ( $node[2] != "" ) {
			return true;
		}
		
		return false;
	}
	
	// Recursive function that prints all sub-nodes for the given main node... for internal use!
	function getSubordinateNodes($mainNode) {
		$subordinates = array();
		foreach ( $this->xml_elements as $ele ) {
			if ( $ele[2] == $mainNode ) {
				$subordinates[] = $ele;
			}
		}
		
		foreach ( $subordinates as $sub ) {
			$this->xml_buffer.= "<".$sub[0]."";
			if ( @is_array($this->xml_attributes["".$sub[0].""]) ) {
				foreach ( $this->xml_attributes["".$sub[0].""] as $key => $value ) {
					$this->xml_buffer.= " $key=\"$value\"";
				}
			}
			
			$this->xml_buffer.= ">";

			$this->getSubordinateNodes($sub[0]);
			$this->xml_buffer.= "".$sub[1]."</".$sub[0].">";
		}
	}
	
	// Writes the resulting xml file to the path specified...
	// Usage: $xmlclass->write_xml_file(XOOPS_ROOT_PATH."/modules/mymodule/cache/xml/myxmlfile.xml");
	function write_xml_file($path) {
		// Processing the header of the file...
		$this->xml_buffer = "<?xml version=\"1.0\" encoding=\"".$this->xml_encoding."\"?>";
		
		// If the programmer has given an xsl file, attach it!
		if ( !empty($this->xml_template) ) {
			$this->xml_buffer.= "<?xml-stylesheet type=\"text/xsl\" href=\"".$this->xml_template."\"?>";
		}
		
		// Let's start with the root node!
		$this->xml_buffer.= "<".$this->root_element.">";
		
		// We now process each root-subordinated node recursively...
		foreach ( $this->xml_elements as $ele ) {
			if ( !$this->isSubordinate($ele) ) {
				$this->xml_buffer.= "<".$ele[0]."";
				// ...and its attributes, if any...
				if ( @is_array($this->xml_attributes["".$ele[0].""]) ) {
					foreach ( $this->xml_attributes["".$ele[0].""] as $key => $value ) {
						$this->xml_buffer.= " $key=\"$value\"";
					}
				}
					
				$this->xml_buffer.= ">";
				// Writing each subnode
				$this->getSubordinateNodes($ele[0]);
				
				// Closing the node...
				$this->xml_buffer.= "".$ele[1]."</".$ele[0].">";
			}
		}
		
		// This is the end of file (root element is closed)...
		$this->xml_buffer.= "</".$this->root_element.">";

		// Now just put all the above into the file...
		// ...just check the extension!
		if ( substr($path, (strlen($path) - 4), 4) != ".xml" ) {
			$path.= ".xml";
		}
				
		if ( !$fp = @fopen($path, "w") ) {
			return false;
		}
		
		if ( @fwrite($fp, $this->xml_buffer) == -1 ) {
			return false;
		}
		
		@fclose($fp);
		return true;		
	}
	
	// Executes an xslt transformation on the result xml file if the xslt library is loaded.
	// Returns the result of the transformation, or false in case of error.
	function xml_transform($xmlfile, $template) {
		if ( function_exists("xslt_process") ) {
			if ( !$file_xml = @file_get_contents(realpath($xmlfile)) ) {
				return false;
			}
			
			if ( !$file_xsl = @file_get_contents(realpath($template)) ) {
				return false;
			}
			
			$xml = $file_xml;
			$xsl = $file_xsl;
			
			$arguments = array("/_xml" => $xml, "/_xsl" => $xsl);
			
			$xh = xslt_create();
			xslt_set_encoding($xh, $this->xml_encoding);
			$result = @xslt_process($xh, "arg:/_xml", "arg:/_xsl", NULL, $arguments);
			echo xslt_error($xh);
			xslt_free($xh);
			if ( !empty($result) ) {
				return $result;
			} else {
				return false;
			}
		} else {
			return false;
		}
	}
	
	// Displays the result of an xml file directly on the browser...
	function get_xml_file($url) {
		@header("Content-type: text/xml");
		@header('Content-Disposition: inline; filename="'.$url.'"');
		@header('Location: '.$url.'');
	}
	
	// Deletes the given xml file...
	function delete_xml_file($localpath) {
		@unlink(realpath($localpath));
	}
	
	// Removes all htmlentities from the given variable
	function code_xml_contents($xml_var) {
	global $myts;
		$result = $myts->makeTareaData4Show($xml_var,1,1,1);
		$result = str_replace("<", "&lt;", $result);
		$result = str_replace(">", "&gt;", $result);
		
		return $result;
	}
	
	// Restores all html entities from the given variable
	function decode_xml_contents($xml_var) {
		$result =  str_replace("&lt;", "<", $xml_var);
		$result =  str_replace("&gt;", ">", $result);
		
		return $result;
	}
}

?>